class AdvancementPlansController < ApplicationController

  # actually, the create has no user input, so no harm can be done here.
  skip_before_action :verify_authenticity_token, only: [:create_from_remote]

  def index
    @grid = AdvancementPlansGrid.new(params[:advancement_plans_grid]) do |scope|
      scope.page(params[:page])
    end
    add_breadcrumb "Förderpläne", :advancement_plans_path
  end

  def show
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take!
    add_breadcrumb "Förderpläne", :advancement_plans_path
    add_breadcrumb @advancement_plan.abbreviation, advancement_plan_path(@advancement_plan.catalog_id)
  end

  def new
    @advancement_plan = AdvancementPlan.new
    authorize!(:update, @advancement_plan)
    add_breadcrumb "Förderpläne", :advancement_plans_path
    add_breadcrumb "Neuer Förderplan"
  end

  def create
    add_breadcrumb "Förderpläne", :advancement_plans_path
    add_breadcrumb "Neuer Förderplan"
    @advancement_plan = AdvancementPlan.new(advancement_plan_params)
    authorize!(:update, @advancement_plan)
    respond_to do |format|
      if @advancement_plan.save
        format.html { redirect_to edit_stock_advancement_plan_path(@advancement_plan.catalog_id),  success: 'Gespeichert'}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :new }
      end
    end
  end

  def edit
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take!
    authorize!(:update, @advancement_plan)
    add_breadcrumb "Förderpläne", :advancement_plans_path
    add_breadcrumb @advancement_plan.abbreviation, advancement_plan_path(@advancement_plan.catalog_id)
  end

  def update
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take!
    @advancement_plan.assign_attributes(advancement_plan_params)
    authorize!(:update, @advancement_plan)
    add_breadcrumb "Förderpläne", :advancement_plans_path
    add_breadcrumb @advancement_plan.abbreviation, advancement_plan_path(@advancement_plan)
    respond_to do |format|
      if @advancement_plan.save
        format.html { redirect_to advancement_plans_path,  success: 'Gespeichert'}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :edit }
      end
    end
  end

  def edit_stock
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take!
    authorize!(:read, @advancement_plan)
    authorize!(:update, Institution)
    add_breadcrumb "Fördermethoden", :advancement_plans_path
    add_breadcrumb @advancement_plan.abbreviation, edit_stock_advancement_plan_path(@advancement_plan.catalog_id)
  end

  def update_stock
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take!
    authorize!(:read, @advancement_plan)
    @advancement_plan.assign_attributes(advancement_plan_stock_params)
    @advancement_plan.advancement_plan_institution_stocks.each do |stock|
      authorize!(:update, stock.institution)
    end
    respond_to do |format|
      if @advancement_plan.save
        format.html { redirect_to edit_stock_advancement_plan_path(@advancement_plan.catalog_id),  success: 'Gespeichert'}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :edit_stock }
      end
    end
  end

  def create_from_remote
    @advancement_plan = AdvancementPlan.where(catalog_id: params[:catalog_id]).take
    if @advancement_plan.present?
      flash[:success] = "Der Katalogeintrag wurde hinzugefügt."
      redirect_to edit_stock_advancement_plan_path @advancement_plan.catalog_id and return
    end
    remote_advancement_plan = CsbApi::AdvancementPlan.includes('advancement-plan-images').where(id: params[:catalog_id]).take(1)
    if remote_advancement_plan.blank?
      flash[:danger] = "Der Katalogeintrag existiert nicht."
      redirect_to  remote_advancement_plans_path and return
    end
    @advancement_plan = remote_advancement_plan.first.build_model
    authorize!(:create, @advancement_plan)
    respond_to do |format|
      if @advancement_plan.save(validate: false)
        format.html { redirect_to edit_stock_advancement_plan_path @advancement_plan.catalog_id,  success: 'Der Katalogeintrag wurde hinzugefügt.'}
      else
        flash.now[:danger] = 'Der zentrale Bibiliothekseintrag ist leider unvollständig und muss noch ergänzt werden.'
        format.html { render :new }
      end
    end
  end

  def destroy
    advancement_plan = AdvancementPlan.where(id: params[:catalog_id]).take!
    advancement_plan.destroy if advancement_plan
    flash[:success] = "Der Eintrag wurde gelöscht."
    redirect_to  advancement_plans_path
  end

  private
  def advancement_plan_stock_params
    params.require(:advancement_plan).permit(
        advancement_plan_institution_stocks_attributes: [
            :id,:institution_id,:advancement_plan_id, :stock
        ]
    )
  end

  def advancement_plan_params
    params.require(:advancement_plan).permit(
        :title,
        :authors,
        :year,
        :subtitle,
        :abbreviation,
        :min_level,
        :max_level,
        :short_description,
        :description,
        :url,
        :age_from,
        :age_to,
        :cover_image,
        :cover_image_cache
    );
  end

end
