class AuthRolesController < ApplicationController

  before_action except: [:all_defined_auth_items,:current_user_grants] do
    authorize!(:update, :system_configuration) unless current_user.super_admin?
  end

  def index
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    @grid = AuthRoleGrid.new(params[:auth_role_grid]) do |scope|
      scope.page(params[:page]).per(25)
    end
  end

  def show
    @auth_role = AuthRole.find(params[:id])
    # authorize!(:show, @auth_role)
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    add_breadcrumb "Rolle anzeigen", auth_role_path(@auth_role)
  end

  def new
    @auth_role = AuthRole.new(user_type: params[:user_type])
    @auth_role.auth_items = AuthItem.where(user_type: params[:user_type],is_default_item:true)
    # authorize!(:new, @auth_role)
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    add_breadcrumb "Rolle erstellen", :new_auth_role_path
  end

  def edit
    @auth_role = AuthRole.find(params[:id])
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    add_breadcrumb "Rolle bearbeiten", edit_auth_role_path(@auth_role)
  end

  def sync
    # authorize!(:sync, AuthRole)
    Ability.sync_auth_item_database
    respond_to do |format|
      format.html { redirect_to auth_roles_path, {success: "Geladen"} }
      format.json { render :index, status: :updated, location: @auth_role }
    end
  end

  def create
    @auth_role = AuthRole.new
    # authorize!(:create, @auth_role)
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    add_breadcrumb "Rolle erstellen", :new_auth_role_path
    @auth_role.assign_attributes(auth_role_params)
    respond_to do |format|
      if @auth_role.save
        format.html { redirect_to auth_roles_path, {success: "Erstellt"} }
        format.json { render :show, status: :updated, location: @auth_role }
      else
        format.html { render :new }
        format.json { render json: @auth_role.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @auth_role = AuthRole.find(params[:id])
    add_breadcrumb "Rechteverwaltung", :auth_roles_path, title: "alle Rollen anzeigen"
    add_breadcrumb "Rolle bearbeiten", edit_auth_role_path(@auth_role)
    @auth_role.assign_attributes(auth_role_params)
    respond_to do |format|
      if @auth_role.save
        format.html { redirect_to edit_auth_role_path(@auth_role), {success: "Geändert"} }
        format.json { render :show, status: :updated, location: @auth_role }
      else
        format.html { render :edit }
        format.json { render json: @auth_role.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    @auth_role = AuthRole.find(params[:id])
    # authorize!(:destroy, @auth_role)
    respond_to do |format|
      if @auth_role.destroy
        format.html { redirect_to auth_roles_path, success: 'gelöscht.' }
        format.json { head :no_content }
      else
        format.html { redirect_to auth_roles_path, danger: 'nicht gelöscht' }
        format.json { render json: @auth_role.errors, status: :unprocessable_entity }
      end
    end
  end

  def current_user_grants
    @user = current_user
    if @user.super_admin?
      @auth_role = AuthRole.new
      @auth_role.auth_item_ids = AuthItem.where(is_su_item: true).pluck(:id)
    end
  end

  def su_grants
    @auth_role = AuthRole.new
    @auth_role.assign_attributes(su_grants_params)
    AuthItem.where(id: @auth_role.auth_item_ids).update_all(is_su_item: true)
    AuthItem.where.not(id: @auth_role.auth_item_ids).update_all(is_su_item: false)
    flash[:success] = "Gesetzt"
    redirect_to current_user_grants_auth_roles_path
  end

  # this lists and executes all available auth items. They are executed with random users, so the results are also random,
  # but at least all ability defining code in items is executed in a proper context, so errors can be seen early and easy
  # this is rather a task for UnitTests, but as always, there is no time for unit testing.
  def all_defined_auth_items
    results = {}
    [Manager,Teacher].each do |user_type|
      user = User.where(identifiable_type: user_type).order('RANDOM()').first
      results[user_type.to_s] = AuthItem.test_run(user,user_type)
    end
    render text: "<pre>#{JSON.pretty_generate(results)}</pre>"
  end

  private
    # Never trust parameters from the scary internet, only allow the white list through.
    def auth_role_params
      params.require(:auth_role).permit(
        :name, :user_type, :description, { auth_item_ids: [] }
      )
    end

  def su_grants_params
    params.require(:auth_role).permit(
        { auth_item_ids: [] }
    )
  end


end