class CareerItemsTeacherController < ApplicationController

  before_action :set_client, only: [:index, :new, :create]

  def index
    # authorize!(:show, CareerItems)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Laufbahn", client_career_items_path(@client), title: "gesamte Laufbahn anzeigen"
    add_breadcrumb "Unterstützung durch sonderpädagogische Fachkraft"

    @career_items_grid = CareerItemsTeacherGrid.new(params[:career_items_teacher_grid]) do |scope|
      scope.where(client_id: params[:client_id]).page(params[:page]).per(25)
    end
  end

  def new
    # authorize!(:teacher_change, @client)
    add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
    add_breadcrumb "#{view_context.show_person(@client)}", dashboard_client_path(@client), title: "Übersicht anzeigen"
    add_breadcrumb "Unterstützung durch sonderpädagogische Fachkraft"

    @career_item = @client.career_items.build({change_type: CareerItem::CHANGE_TYPE_TEACHER})

    # alle in der aktuellen Einrichtung verfügbaren Lehrpersonen
    # gruppiert nach Kategorie
    # mit ergänzender Info zu derzeitiger Tätigkeit
    @available_teachers = {}
    if @client.teacher.present?
      @available_teachers['keine Unterstützung'] = [["die aktuelle Unterstützung wird beendet", nil]]
    end
    TeacherCategory.all.each do |tc|
      @available_teachers[tc.name] = []
      @client.institution.teachers.where(teacher_category: tc).each do |teacher| 
        @available_teachers[tc.name] <<  [
          teacher == @client.teacher ? view_context.show_person(teacher) + " (aktuell)" : view_context.show_person(teacher),
          teacher.id, 
          {'data-subtext': "#{teacher.current_client_group.try(:name)}"}
        ]
      end
    end
  end

  def create
    # authorize!(:teacher_change, @client)
    @client.assign_attributes(client_params)
    respond_to do |format|
      if @client.save
        success = "Die Unterstützung durch die sonderpädagogische Fachkraft wurde gewechselt. Die Laufbahn von " + view_context.show_person(@client) + " wurde entsprechend aktualisiert."
        format.html { redirect_to dashboard_client_path(@client), success: success }
      else
        format.html { render :new }
      end
    end
  end


  private
    # Use callbacks to share common setup or constraints between actions.

    def client_params
      params.require(:client).permit(:teacher_id, career_items_attributes: [:change_type, :description])
    end

    def set_client
      @client = Client.find(params[:client_id])
    end

end
