class ClientGroupsController < ApplicationController

  def index
    authorize!(:read, ClientGroup)
    @institution = Institution.where(id: params[:institution_id]).where(institution_type: [Institution::TYPE_EDUCATIONAL,Institution::TYPE_SCHOOL]).take!
    add_breadcrumb "Schulen" # und Bildungseinrichtungen
    add_breadcrumb "#{@institution.name}", institution_path(@institution), title: "Einrichtung anzeigen"
    add_breadcrumb "Alle Klassen"
    @grid = ClientGroupGrid.new(params[:client_group_grid]) do |scope|
      scope.accessible_by(current_ability).where(institution_id: @institution.id).page(params[:page]).per(25)
    end
  end

  def show
    @client_group = find_client_group
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "Klasse #{@client_group.name}", client_group_path(@client_group), title: "Details anzeigen"
    @latest_portfolio_items = @client_group.portfolio_items.order(start: :desc, updated_at: :desc).includes([:portfolio_specialization,:portfolio_step])
    
  end

  def select_portfolio_item_date
    @client_group = ClientGroup.where( id: params[:client_group_id]).take!
    @portfolio_item = ::Portfolio::Item.new({portfolio: @client_group.running_portfolio})
    authorize!(:create, ::Portfolio::Item)
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "Klasse #{@client_group.name}", client_group_path(@client_group), title: "Details anzeigen"
    add_breadcrumb "neuer Eintrag im Portfolio", select_portfolio_item_date_client_groups_path()
  end

  def select_portfolio_item_step
    @client_group = ClientGroup.where( id: params[:client_group][:id]).take!
    @portfolio_item = ::Portfolio::Item.new({portfolio: @client_group.running_portfolio}.merge(params.require(:portfolio_item).permit( :start)))
    authorize!(:create, ::Portfolio::Item)
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "Klasse #{@client_group.name}", client_group_path(@client_group), title: "Details anzeigen"
    add_breadcrumb "neuer Eintrag im Portfolio", select_portfolio_item_step_client_groups_path()
  end

  def select_institution
    @client_group = ClientGroup.new(params.fetch(:client_group, {}).permit(:institution_id))
    add_breadcrumb "Klasse erstellen", :new_client_group_path
    unless Institution.education_facilities.any?
      flash[:info] = "Es sind noch keine Schulen vorhanden. Bitte anlegen."
      return redirect_to select_new_institution_path(institution_type: Institution::TYPE_SCHOOL)
    end
    authorize!(:create, ClientGroup)
    @available_semesters = build_available_semesters
  end

  def new
    if params.has_key?(:client_group)
      @client_group = ClientGroup.new(client_group_params)
    else
      @client_group = ClientGroup.new
    end
    unless @client_group.semester.present? && @client_group.institution.present?
      return redirect_to :select_institution_new_client_group
    end
    authorize!(:create, ClientGroup)
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "neue Klasse", :new_client_group_path
    @available_teachers = build_available_teachers
    @available_semesters = build_available_semesters
  end

  # GET /client_groups/1/edit
  def edit
    @client_group = find_client_group
    authorize!(:update, @client_group)
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "Klasse #{@client_group.name}", client_group_path(@client_group), title: "Details anzeigen"
    add_breadcrumb "bearbeiten", edit_client_group_path(@client_group)
    @available_teachers = build_available_teachers
    @available_semesters = @client_group.semester
  end

  def create
    @client_group = ClientGroup.new(client_group_params)
    authorize!(:create, @client_group)
    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "neue Klasse", :new_client_group_path    
    @available_teachers = build_available_teachers
    @available_semesters = build_available_semesters
    respond_to do |format|
      if @client_group.save
        format.html { redirect_to @client_group.institution, success: 'Die Klasse wurde erfolgreich angelegt.' }
      else
        format.html { render :new }
      end
    end
  end

  # PATCH/PUT /client_groups/1
  # PATCH/PUT /client_groups/1.json
  def update
    @client_group = find_client_group

    add_breadcrumb "#{@client_group.institution.name}", institution_path(@client_group.institution)
    add_breadcrumb "Klasse #{@client_group.name}", client_group_path(@client_group), title: "Details anzeigen"
    add_breadcrumb "bearbeiten", edit_client_group_path(@client_group)    
    @available_teachers = build_available_teachers
    @available_semesters = build_available_semesters
    @client_group.assign_attributes(client_group_params)
    authorize!(:update, @client_group)
    notice = "Die Klasse wurde erfolgreich aktualisiert."
    respond_to do |format|
      if @client_group.save
        format.html { redirect_to @client_group.institution, success: notice }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /client_groups/1
  # DELETE /client_groups/1.json
  def destroy
    @client_group = find_client_group
    authorize!(:destroy, @client_group)
    respond_to do |format|
      if @client_group.destroy
        format.html { redirect_to institution_url(@client_group.institution), success: 'Die Klasse wurde erfolgreich gelöscht.' }
      else
        format.html { redirect_to institution_url(@client_group.institution), danger: 'Die Klasse konnte nicht gelöscht werden. ' + @client_group.errors[:base].first }
      end
    end
  end


  private
    # Use callbacks to share common setup or constraints between actions.
    def find_client_group
      @client_group = ClientGroup.find(params[:id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def client_group_params
      params.require(:client_group).permit(
          :name, :description, :teacher_id, :institution_id, :normal_education_hours, :special_education_hours,
          :portfolio_source_client_group_id, # this is for assigning another client_group running portfolio to this client group
          :clients_total, :semester_id, client_ids: [], educational_course_rule_ids: []
      )
    end

    def build_available_semesters
      Semester.following_and_current
    end

    def build_available_teachers
      @client_group.unassigned_teachers
    end

end
