class ClientSupporterRelationsController < ApplicationController

  def new
    # wenn ausgewählt zum Formular für neue Externe Begleitperson wechseln
    if params.has_key?(:client_supporter_relation)
      if client_supporter_relation_params[:supporter_id] == 'new'
        session[:client_supporter_relation_client_id] = params[:client_id]
        flash[:info] = "Hier kann eine neue externe Begleitperson eingetragen werden. Diese ist nach erfolgreicher Speicherung zur Auswahl verfügbar."
        return redirect_to new_supporter_path
      end
    end
    @client = find_client
    authorize!(:update, @client)
    @available_supporters = build_available_supporters
    set_default_breadcrumbs(@client)
    add_breadcrumb "neu"
    if params.has_key?(:client_supporter_relation)
      @client_supporter_relation = @client.client_supporter_relations.build(client_supporter_relation_params)  
    else
      @client_supporter_relation = @client.client_supporter_relations.build()
    end
    build_documents
  end

  def edit
    @client_supporter_relation = find_client_supporter_relation
    authorize!(:update, @client_supporter_relation.client)
    @available_supporters = build_available_supporters    
    set_default_breadcrumbs(@client_supporter_relation.client)
    build_documents
  end

  def create
    @client = find_client
    authorize!(:update, @client)
    @client_supporter_relation = @client.client_supporter_relations.build(client_supporter_relation_params)
    @available_supporters = build_available_supporters
    set_default_breadcrumbs(@client)
    # @client.assign_attributes(client_params) this calls not existing method client_params, what was the intention?
    respond_to do |format|
      if @client_supporter_relation.save
        format.html { redirect_to dashboard_client_url(@client_supporter_relation.client), success: 'Externe Begleitung wurde erfolgreich gespeichert.' }
      else
        build_documents # only on error
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client_supporter_relation.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :new }
      end
    end
  end

  def update
    @client_supporter_relation = find_client_supporter_relation
    authorize!(:update, @client_supporter_relation.client)
    @available_supporters = build_available_supporters
    set_default_breadcrumbs(@client_supporter_relation.client)
    @client_supporter_relation.assign_attributes(client_supporter_relation_params)
    respond_to do |format|
      if @client_supporter_relation.save
        format.html { redirect_to edit_client_supporter_relation_url(@client_supporter_relation), success: 'Externe Begleitung wurde erfolgreich aktualisiert.' }
      else
        build_documents # only on error
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @client_supporter_relation.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit }
      end
    end
  end

  # DELETE /client_supporter_relations/1
  def destroy
    # authorize!(:destroy, ClientSupporterRelation)
    @client_supporter_relation = find_client_supporter_relation
    authorize!(:update, @client_supporter_relation.client)
    respond_to do |format|
      if @client_supporter_relation.destroy
        format.html { redirect_to dashboard_client_url(@client_supporter_relation.client), success: 'Externe Begleitung wurde erfolgreich beendet.' }
      else
        format.html { redirect_to dashboard_client_url(@client_supporter_relation.client), danger: 'Externe Begleitung konnte nicht beendet werden. ' + @client_supporter_relation.errors[:base].first }
      end
    end
  end

  def destroy_document
    @client_supporter_relation = ClientSupporterRelation.find(params[:client_supporter_relation_id])
    authorize!(:update, @client_supporter_relation.client)
    @document = @client_supporter_relation.documents.find(params[:id])
    respond_to do |format|
      if @document.destroy!
        format.html { redirect_to edit_client_supporter_relation_url(@client_supporter_relation), success: "Das Dokument wurde gelöscht." }
      else
        format.html { redirect_to edit_client_supporter_relation_url(@client_supporter_relation), danger: "nicht möglich" }
      end
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def find_client_supporter_relation
      ClientSupporterRelation.find(params[:id])
    end

    def find_client
      Client.find(params[:client_id])
    end

    def set_default_breadcrumbs(client)
      add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
      add_breadcrumb "#{view_context.show_person(client)}", dashboard_client_path(client), title: "Übersicht anzeigen"
      add_breadcrumb "externe Begleitung"
    end

    def build_available_supporters
      available_supporters = {}
      Supporter.all.each do |supporter|
        option = [view_context.show_person(supporter), supporter.id]
        key = (institution = supporter.institution) ? institution.name : "ohne Einrichtung"
        if available_supporters.has_key?(key) 
          available_supporters[key] << option
        else
          available_supporters[key] = [option]
        end
      end
      available_supporters
    end

    def build_documents    
      unless @client_supporter_relation.documents.select { |document| document[:category] == Document::ENUM_CATEGORY_CONFIDENTIALITY_DOCUMENT }.any?
        @client_supporter_relation.documents.build(category: Document::ENUM_CATEGORY_CONFIDENTIALITY_DOCUMENT)
      end
      if @client_supporter_relation.documents.select { |document| document[:category] == Document::ENUM_CATEGORY_OTHER_DOCUMENT && document.new_record? }.any?
        @other_document = @client_supporter_relation.documents.select { |document| document[:category] == Document::ENUM_CATEGORY_OTHER_DOCUMENT && document.new_record? }.last
      else
         @other_document = @client_supporter_relation.documents.build(category: Document::ENUM_CATEGORY_OTHER_DOCUMENT)
      end
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def client_supporter_relation_params
      params.require(:client_supporter_relation).permit(
        :client_id, :supporter_id, :description,
        documents_attributes: [
            :id, :file, :category,
            :file_cache,
            :_destroy
        ],
      )
    end

end
