class PortfolioItemsController < ApplicationController
  include PdfConcern

  def show

    @portfolio_item = ::Portfolio::Item.where(id: params[:id]).take!
    abort @portfolio_item.cannot_be_closed_reason.inspect
    authorize!(:read, @portfolio_item)
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "#{@portfolio_item.portfolio_step.name} vom #{l(@portfolio_item.start, format: :short)}", portfolio_item_path(@portfolio_item)
    respond_to do |format|
      format.html
      format.pdf { render_pdf(
          "#{@portfolio_item.portfolio_step.name}-#{@portfolio_item.portfolio_subject.name}-#{l(@portfolio_item.start,format: :short)}.pdf",
          :show,
          title: "#{@portfolio_item.portfolio_step.name}_#{@portfolio_item.portfolio_subject.name}",
          header_left: "#{@portfolio_item.portfolio_step.name} - #{@portfolio_item.portfolio_subject.name}",
          header_right: "#{l(@portfolio_item.start,format: :short)}"
        )
      }
    end
  end



  def evaluations
    @portfolio_item = ::Portfolio::Item.where(id: params[:id]).take!
    authorize!(:read, @portfolio_item)
    @portfolio_item_targets = ::Portfolio::ItemTarget.where(portfolio_item_id: params[:id], evaluate: true).all
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "#{@portfolio_item.portfolio_step.name} vom #{l(@portfolio_item.start, format: :short)}", portfolio_item_path(@portfolio_item)
    add_breadcrumb "Evaluationen"
  end

  def new
    @portfolio_item = ::Portfolio::Item.new(params.require(:portfolio_item).permit( :start, :portfolio_id, :portfolio_step_id))
    authorize!(:create, ::Portfolio::Item)
    @portfolio_item.edited_by = current_user.person
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "neuer Eintrag im Portfolio", new_portfolio_item_path()
  end

  def edit
    @portfolio_item = ::Portfolio::Item.where(id: params[:id]).take!
    authorize!(:update, @portfolio_item)
    @portfolio_item.edited_by = current_user.person
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "#{@portfolio_item.portfolio_step.name} vom #{l(@portfolio_item.start, format: :short)}", portfolio_item_path(@portfolio_item)
    add_breadcrumb "bearbeiten", edit_portfolio_item_path(@portfolio_item)
  end

  def create
    @portfolio_item = ::Portfolio::Item.new(create_portfolio_item_params)
    authorize!(:create, @portfolio_item)
    @portfolio_item.edited_by = current_user.person
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "neuer Eintrag im Portfolio"
    respond_to do |format|
      if  @portfolio_item.save
        format.html { redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  success:  "Gespeichert."}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :new  }
      end
    end
  end

  def update
    @portfolio_item = ::Portfolio::Item.where(id: params[:id]).take!
    authorize!(:update, @portfolio_item)
    @portfolio_item.assign_attributes(@portfolio_item.closed? ? closed_portfolio_item_params : update_portfolio_item_params)
    @portfolio_item.edited_by = current_user.person
    subjects_breadcrumb(@portfolio_item)
    subject_breadcrumb(@portfolio_item)
    add_breadcrumb "#{@portfolio_item.portfolio_step.name} vom #{l(@portfolio_item.start, format: :short)}", portfolio_item_path(@portfolio_item)
    add_breadcrumb "bearbeiten"
    respond_to do |format|
      if  @portfolio_item.save
        format.html { redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  success:  "Gespeichert."}
      else
        flash.now[:danger] = 'Bitte prüfen Sie Ihre Eingaben'
        format.html { render :edit }
      end
    end
  end

  def close
    @portfolio_item = ::Portfolio::Item.where(id: params[:id]).take!
    authorize!(:update, @portfolio_item)
    respond_to do |format|
      if @portfolio_item.can_be_closed?
        @portfolio_item.close
        format.html { redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  success:  "Eintrag abgeschlossen." }
      else
        format.html {  redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  danger:  "Eintrag kann noch nicht abgeschlossen werden. #{@portfolio_item.cannot_be_closed_reason}"  }
      end
    end
  end

  def destroy
    @portfolio_item = ::Portfolio::Item.where( id: params[:id]).take!
    authorize!(:destroy, @portfolio_item)
    respond_to do |format|
      if @portfolio_item.can_be_deleted? && @portfolio_item.destroy

        format.html { redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  success:  "Eintrag gelöscht." }
      else
        format.html {  redirect_to @portfolio_item.portfolio_subject.redirect_after_item_create,  danger:  "Eintrag kann nicht gelöscht werden. #{@portfolio_item.cannot_be_deleted_reason}"  }
      end
    end
  end

  private
    def create_portfolio_item_params
      params.require(:portfolio_item).permit(
          :portfolio_step_id, :portfolio_specialization_id, :start, :content, :target_agreement,
          :portfolio_step_result_id, :portfolio_id,
          :parent_teacher_conference_date, :management_conference_date, :subject_involved,
          :content_is_attached,:level,:creator_involved,
          portfolio_documents_attributes: [
              :id,:file_name, :file_name_cache,:portfolio_document_type_id, :document_type,:_destroy
          ],
          involved_teacher_ids: [], involved_guardian_ids: [],involved_supporter_ids: [],
          target_agreements_attributes: [
              :id,:content,:result_comment,:evaluate,:_destroy,
              evaluation_attributes: [
                  :id,:end_date, :additions, :start_date, :result_scheme,:subject_involved,
                  involved_teacher_ids: [],
                  involved_supporter_ids: [],
                  involved_guardian_ids: [],
                  evaluation_involved_persons_attributes: [
                      :id,:name
                  ],
                  portfolio_item_evaluation_dbr_attributes:[
                      :id,:range_begin, :range_end, :setting_social, :setting_time, :number_of_data_points,
                      characteristics_attributes: [
                          :id,:name,:increase_is_positive
                      ]
                  ]
              ],
          ],
          advancement_plans_attributes: [
              :id,:advancement_plan_id,:catalog_id,:begin,:content,:_destroy
          ],
          measurements_attributes: [
              :id,:begin,:content,:_destroy
          ],
          diagnostics_attributes: [
              :id,:diagnostic_id,:catalog_id,:begin,:content,:results_closed,:_destroy,
          ],
          diagnostic_measurements_attributes: [
              :id,:begin,:content,:results_closed,:_destroy,
          ],
          class_diagnostics_attributes: [
              :id,:remote_id,:begin,:content,:results_closed,:_destroy,
          ],
          portfolio_item_involved_people_attributes: [
              :id,:name,:_destroy
          ]
      )
    end

    def update_portfolio_item_params
      params.require(:portfolio_item).permit(
          :portfolio_specialization_id, :content, :target_agreement,
          #:parent_teacher_conference, :management_conference,
          :portfolio_step_result_id,
          :parent_teacher_conference_date, :management_conference_date, :subject_involved,
          :content_is_attached,:level,:creator_involved,
          portfolio_documents_attributes: [
              :id,:file_name, :file_name_cache,:portfolio_document_type_id, :document_type,:_destroy
          ],
          involved_teacher_ids: [], involved_guardian_ids: [],involved_supporter_ids: [],
          target_agreements_attributes: [
              :id,:content,:result_comment,:evaluate,:_destroy,
              evaluation_attributes: [
                  :id,:end_date, :additions, :start_date, :subject_involved, :result_scheme,
                  involved_teacher_ids: [],
                  involved_supporter_ids: [],
                  involved_guardian_ids: [],
                  evaluation_involved_persons_attributes: [
                      :id,:name
                  ],
                  portfolio_item_evaluation_dbr_attributes:[
                      :id,:range_begin, :range_end, :setting_social, :setting_time, :number_of_data_points,
                      characteristics_attributes: [
                          :id,:name,:increase_is_positive,:_destroy
                      ]
                  ]
              ],
          ],
          advancement_plans_attributes: [
              :id,:advancement_plan_id,:catalog_id,:begin,:content,:_destroy
          ],
          measurements_attributes: [
              :id,:begin,:content,:_destroy
          ],
          diagnostics_attributes: [
              :id,:diagnostic_id,:catalog_id,:begin,:content,:results_closed,:_destroy,
          ],
          diagnostic_measurements_attributes: [
              :id,:begin,:content,:results_closed,:_destroy,
          ],
          class_diagnostics_attributes: [
              :id,:remote_id,:begin,:content,:results_closed,:_destroy,
          ],
          portfolio_item_involved_people_attributes: [
              :id,:name,:_destroy
          ]
      )
    end

    def closed_portfolio_item_params
      params.require(:portfolio_item).permit(
          :portfolio_specialization_id, :content, :target_agreement,
          :content_is_attached,
          portfolio_documents_attributes: [
              :id,:file_name,:file_name_cache, :portfolio_document_type_id, :document_type
          ],
          target_agreements_attributes: [
              :id,:content,:result_comment,
              evaluation_attributes: [
                  :id, :additions
              ],
          ],
          advancement_plans_attributes: [
              :id,:content
          ],
          measurements_attributes: [
              :id,:content
          ],
          diagnostics_attributes: [
              :id,:content
          ],
          diagnostic_measurements_attributes: [
              :id,:content
          ],
          class_diagnostics_attributes: [
              :id,:content
          ],
          portfolio_item_involved_people_attributes: [
              :id,:name,:_destroy
          ]
      )
    end

    def subjects_breadcrumb(portfolio_item)
      case portfolio_item.portfolio.subject.class.name
        when 'Client'
          add_breadcrumb "SchülerInnen", :clients_path, title: "alle SchülerInnen anzeigen"
        when 'ClientGroup'
          add_breadcrumb "Einrichtung", institution_path(portfolio_item.portfolio_subject.institution), title: "#{portfolio_item.portfolio_subject.institution.name}"
      end

    end

    def subject_breadcrumb(portfolio_item)
      case portfolio_item.portfolio.subject.class.name
        when 'Client'
          add_breadcrumb "#{view_context.show_person(portfolio_item.portfolio.subject)}", dashboard_client_path(portfolio_item.portfolio.subject), title: "Übersicht anzeigen"
        when 'ClientGroup'
          add_breadcrumb "Klasse #{portfolio_item.portfolio_subject.name}", client_group_path(portfolio_item.portfolio.subject), title: "Klassenübersicht anzeigen"
      end
    end

end
