class PortfolioSpecializationsController < ApplicationController

  before_action :set_portfolio_specialization, only: [:show, :edit, :update, :destroy]

  before_action do
    authorize!(:update, :system_configuration)
  end

  def index
    @portfolio_specializations_grid = PortfolioSpecializationsGrid.new(params[:portfolio_specializations_grid]) do |scope|
      scope.page(params[:page]).per(5)
    end
    add_breadcrumb "Systemeinstellungen", configurations_path
    add_breadcrumb "Förderschwerpunkte", portfolio_specializations_path
  end


  def new
    @portfolio_specialization = PortfolioSpecialization.new
    add_breadcrumb "Systemeinstellungen", configurations_path
    add_breadcrumb "Förderschwerpunkte", portfolio_specializations_path
    add_breadcrumb "Neu erstellen", new_portfolio_specialization_path
  end

  def edit

    add_breadcrumb "Systemeinstellungen", configurations_path
    add_breadcrumb "Förderschwerpunkte", portfolio_specializations_path
    add_breadcrumb "#{@portfolio_specialization.name} bearbeiten", edit_portfolio_specialization_path(@portfolio_specialization)
  end

  def create
    @portfolio_specialization = PortfolioSpecialization.new(portfolio_specialization_params)
    respond_to do |format|
      if @portfolio_specialization.save
        format.html { redirect_to portfolio_specializations_path, success: 'Gespeichert' }
      else
        format.html { render :new }
      end
    end
  end

  def update
    respond_to do |format|
      if @portfolio_specialization.update(portfolio_specialization_params)
        format.html { redirect_to portfolio_specializations_path, success: 'Gespeichert' }
      else
        format.html { render :edit }
      end
    end
  end

  def destroy
    respond_to do |format|
      if @portfolio_specialization.is_destructable?
        @portfolio_specialization.destroy
        format.html { redirect_to portfolio_specializations_path, success: 'Gelöscht' }
      else
        format.html { redirect_to portfolio_specializations_path, warning: 'Kann nicht gelöscht werden.' }
      end
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_portfolio_specialization
      @portfolio_specialization = PortfolioSpecialization.find(params[:id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def portfolio_specialization_params
      params.require(:portfolio_specialization).permit(:name, :abbreviation)
    end
end
