class SupportersController < ApplicationController

  def index
    authorize!(:read, Supporter)
    add_breadcrumb "Externe Personen", :supporters_path
    @grid = SupporterGrid.new(params[:supporter_grid]) do |scope|
      scope.accessible_by(current_ability).page(params[:page]).per(25)
    end
  end

  def show
    @supporter = find_supporter
    authorize!(:read, @supporter)
    add_breadcrumb "Externe Personen", :supporters_path
    add_breadcrumb "#{view_context.show_person(@supporter)}", supporter_path(@supporter), title: "Details anzeigen"
  end

  def new
    authorize!(:create, Supporter)
    @supporter = Supporter.new()
    if params.has_key?(:supporter)
      @supporter.assign_attributes(supporter_params)
    end
    add_breadcrumb "Externe Personen", :supporters_path
    add_breadcrumb "neu", :new_supporter_path
  end

  def edit
    @supporter = find_supporter
    authorize!(:update, @supporter)
    if params.has_key?(:supporter)
      @supporter.assign_attributes(supporter_params)
    end
    add_breadcrumb "Externe Personen", :supporters_path
    add_breadcrumb "#{view_context.show_person(@supporter)}", supporter_path(@supporter), title: "#{view_context.show_person(@supporter)} anzeigen"
    add_breadcrumb "bearbeiten", edit_supporter_path(@supporter)
  end

  def create
    add_breadcrumb "Externe Personen", :supporters_path
    add_breadcrumb "neu", :new_supporter_path
    @supporter = Supporter.new(supporter_params)
    authorize!(:create, @supporter)
    respond_to do |format|
      if @supporter.save
        # wenn ausgewählt zum Formular für neue Externe Einrichtung wechseln
        if @supporter.institution_choice == '3'
          session[:supporter_id] = @supporter.id
          flash[:info] = "Erfolgreich gespeichert. Hier kann nun eine neue externe Einrichtung eingetragen werden. Diese ist nach erfolgreicher Speicherung zur Auswahl verfügbar."
          redirect_to new_institution_path(institution_type: Institution::TYPE_EXTERN) and return
        end
        # bei Bedarf zum Formular für neue Externe Begleitung wechseln
        if session.has_key?(:client_supporter_relation_client_id)
          flash[:info] = "Erfolgreich gespeichert. Nun kann die neue externe Begleitperson ausgewählt werden."
          path = new_client_client_supporter_relation_path(client_id: session[:client_supporter_relation_client_id], client_supporter_relation: {supporter_id: @supporter.id} )
          session.delete(:client_supporter_relation_client_id)
          redirect_to path and return
        end
        if @supporter.institution_choice == '2'
          flash[:success] = view_context.show_person(@supporter) + " wurde erfolgreich gespeichert."
          redirect_to institution_path(@supporter.institution) and return
        end
        format.html { redirect_to @supporter, success: view_context.show_person(@supporter) + " wurde erfolgreich gespeichert." }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @supporter.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :new }
      end
    end
  end

  def update
    @supporter = find_supporter
    authorize!(:update, @supporter)
    add_breadcrumb "Externe Personen", :supporters_path
    add_breadcrumb "#{view_context.show_person(@supporter)}", supporter_path(@supporter), title: "#{view_context.show_person(@supporter)} anzeigen"
    add_breadcrumb "bearbeiten", edit_supporter_path(@supporter)    
    respond_to do |format|
      if @supporter.update(supporter_params)
        # wenn ausgewählt zum Formular für neue Externe Einrichtung wechseln
        if @supporter.institution_choice == '3'
          session[:supporter_id] = @supporter.id
          flash[:info] = "Erfolgreich gespeichert. Hier kann nun eine neue externe Einrichtung eingetragen werden. Diese ist nach erfolgreicher Speicherung zur Auswahl verfügbar."
          redirect_to new_institution_path(institution_type: Institution::TYPE_EXTERN) and return
        end
        format.html { redirect_to @supporter, success: view_context.show_person(@supporter) + " wurde erfolgreich aktualisiert." }
      else
        error_message = 'Bitte prüfen Sie Ihre Eingaben.'
        @supporter.errors[:base].each do |message|
          error_message += '<br/>' + message
        end
        flash.now[:danger] = error_message.html_safe
        format.html { render :edit }
      end
    end
  end

  def destroy
    @supporter = find_supporter
    authorize!(:destroy, @supporter)
    @supporter.destroy
    respond_to do |format|
      format.html { redirect_to supporters_url, success: 'Supporter was successfully destroyed.' }
    end
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def find_supporter
      Supporter.find(params[:id])
    end


    # Never trust parameters from the scary internet, only allow the white list through.
    def supporter_params
      params.require(:supporter).permit(
        :title, :first_name, :last_name, :gender, :email,
        :phone, :mobile, :fax, :description,
        :position, :institution_id,
        :institution_choice)
    end
end
