class UserPagesController < ApplicationController

  def dashboard
    # authorization is not done here, as everyone can access his own dashboard
    if current_user.is_teacher?
      teacher_dashboard( current_user.person ) and return
    end
    if current_user.is_manager?
      redirect_to institutions_path and return
    end
    render :file => "#{Rails.root}/public/404.html",  :status => 404
  end

  def notifications
    # authorization is not done here, as everyone can access his own notifications
    if current_user.is_teacher?
      teacher_notifications( current_user.person ) and return
    end
    if current_user.is_manager?
      redirect_to notifications_path(params) and return
    end
    render :file => "#{Rails.root}/public/404.html",  :status => 404
  end

  private
  def teacher_notifications(teacher)
    @notificationable_type = params[:notificationable]
    @teacher = teacher
    @grid = NotificationGrid.new(params[:notification_grid]) do |scope|
      scope = scope.only_type(@notificationable_type) if @notificationable_type
      scope.notifications_for_teacher(teacher.id).page(params[:page]).per(10)
    end
    render 'teacher_notifications'
  end

  def teacher_dashboard(teacher)
    @teacher = teacher
    @latest_portfolio_items = teacher.latest_portfolio_items.includes(:involved_guardians,:involved_teachers)
    @associated_clients = @teacher.associated_clients.includes(:client_group,:guardians)
    @notifications = @teacher.system_notifications
    @client_groups = @teacher.associated_client_groups.joins(:semester).where(semesters: {id: Semester.following_and_current.ids})
    @library_suggestions = @associated_clients.collect do |c|
      c.diagnostic_suggestions
    end.flatten
    render 'teacher_dashboard'
  end





end
