class NotificationGrid < BaseGrid

  scope do
    Notification.all
  end

  filter(:content, :string, header: 'Inhalt') do |value|
    self.where(["notifications.content ILIKE ?",  "%#{value}%"])
  end

  filter(:prio, :enum, header: 'Priorität', :select => [['niedrig',0],['mittel',1], ['hoch',2]])

  filter(:overdue, :xboolean, header: 'Überfällig?') do |value|
      self.where(["notifications.due_at <= ?", DateTime.now])  if value
  end

  filter(:institution, :string, header: 'Einrichtung') do |value,scope,grid|
    if /^\d+$/.match(value)
      scope.where(['notifications.institution_id=? ',value])
    else
      scope.where(['notifications.institution_id IN (SELECT id FROM institutions WHERE name ILIKE ?) ', "%#{value}%"])
    end
  end

  # this filter should not be shown, it is skipped in notification index view where the filter form is rendered
  filter(:teacher_id, :integer, header: 'betrifft') do |value,scope|
    scope.notifications_for_teacher(value)
  end

  # This maps given ids of client group or instituion on the readable names
  def filter_value_as_string(name)
   value = super(name)
   name = name.name if name.respond_to? :name # weird thing, seems to be only symbol sometimes, then Filter class other time
   case name
     when :institution
       value = Institution.where(id: value).pluck(:name).first if /^\d+$/.match(value)
     when :client_group
       value = ClientGroup.where(id: value).pluck(:name).first if /^\d+$/.match(value)
     when :teacher_id
       if /^\d+$/.match(value)
         value = Teacher.where(id: value).first
         value = value.full_name
       end
   end
   return value
  end


  # this filter allows following searches:
  # An integer is passed:
  # - the institution is also set: searches for the client_group_id at the selected institution
  # - the institution is not set: searches for the client_group_id globally
  # A string is passed:
  # - the institution is also set: searches for the client_group name at the selected institution
  # - the institution is not set: does not apply
  filter(:client_group, :string, header: 'Klasse in der Einrichtung') do |value, scope, grid|
    if /^\d+$/.match(value)
      scope.where([
          '((notifications.notificationable_type=? AND notifications.notificationable_id=?) OR
          (notifications.client_id IN (SELECT id FROM clients WHERE client_group_id=?)))',
          'ClientGroup',value,value
      ])
    else
      if grid.institution.present?
        if /^\d+$/.match(grid.institution)
          value = ClientGroup.where(name: value, institution_id: grid.institution).pluck(:id).first
        else
          value = ClientGroup.where(name: value, institution_id: Institution.where('name LIKE ?', "%#{grid.institution}%" ).pluck(:id).first).pluck(:id).first
        end
        scope.where([
            '((notifications.notificationable_type=? AND notifications.notificationable_id=?) OR
            (notifications.client_id IN (SELECT id FROM clients  WHERE client_group_id=?)))',
            'ClientGroup',value,value
        ])
      end
    end
  end


  # Here, the columns are solely for sorting purpose
  column(:date, header: 'Datum', order: 'due_at')
  column(:prio, header: 'Priorität', order: 'prio')

end