class SupporterGrid < BaseGrid

  scope do
    Supporter
      .includes(client_supporter_relations: :client)
      .includes(:institution)
      .references(client_supporter_relations: :client)
      .order('supporters.last_name DESC')
      .distinct
  end

  filter(:name, :string, header: "Name") do |value, scope|
     value.split.inject(scope) do |_scope,part|
       _scope.where("supporters.first_name ILIKE '%#{part}%' OR supporters.last_name ILIKE '%#{part}%'")
     end
  end

  filter(:client_name, :string, header: "betreute SchülerInnen") do |value, scope|
    value.split.inject(scope) do |_scope,part|
      _scope.where("clients.first_name ILIKE '%#{part}%' OR clients.last_name ILIKE '%#{part}%'")
    end
  end

  filter(:position, :string, header: "Position") do |value, scope|
      scope.where("position ILIKE '%#{value}%'")
  end

  filter(:description, :string, header: "Notizen") do |value, scope|
    scope.where("description ILIKE '%#{value}%'")
  end

  filter(:institution, :string, header: "Einrichtung") do |value, scope|
    scope.where("institutions.name ILIKE '%#{value}%'")
  end


  column(:name, header: "Name", :html => true) do |supporter|
    show_supporter(supporter)
  end

  column(:institution, header: "Einrichtung", html: true)  do |supporter|
    show_institution(supporter.institution)
  end

  column(:clients, header: "betreute SchülerInnen", html: true)  do |supporter|
    show_persons(supporter.clients)
  end

  column(:email, header: "Email", :html => true)

  column(:phone, header: "Telefon", :html => true)

  column(:mobile, header: "Mobil", :html => true)

  column(:description, header: "Notizen", html: true) do |supporter|

    if description = supporter.description

      id1 = "supporter#{supporter.id}DescriptionModal"
      id2 = "supporter#{supporter.id}DescriptionModalLabel"

      html = ""
      html += "<div class='modal' id='#{id1}' tabindex='-1' role='dialog' aria-labelledby='#{id2}'>
          <div class='modal-dialog' role='document'>
            <div class='modal-content'>
              <div class='modal-header'>
                <button type='button' class='close' data-dismiss='modal' aria-label='Schließen'>
                  <span aria-hidden='true'>&times;</span>
                </button>
                <h4 class='modal-title' id='#{id2}'>Notizen</h4>
              </div>
              <div class='modal-body'>
                #{description}
              </div>
            </div>
          </div>
        </div>"
      html += truncate(description, length: 50) {
        button_tag("(weiterlesen)", {
            class: "btn-link",
            type: "button",
            title: "weitere Informationen anzeigen",
            'data-toggle': 'modal',
            'data-target': '#'+id1,
        })
      }

      html.html_safe
    end
  end

  column(:actions,  header: '', :html => true) do |i|
    _wrp = 0
    delete_button = ""
    update_button = ""
    _wrp += 1
    read_button = "<a href='#{supporter_path(i)}' role='button' class='btn btn-success'>
          <i class='glyphicon glyphicon-eye-open'></i>
        </a>"
    if can?(:update,i)
      _wrp += 1
      update_button = " <a href='#{edit_supporter_path(i)}' role='button' class='btn btn-success btn-sm'>
            <i class='glyphicon glyphicon-pencil'></i>
          </a>"
    end
    if can?(:destroy,i)
      _wrp += 1
      delete_button = " #{ link_to  supporter_path(i), data: {:confirm => 'Sicher?'},
                                    role: 'button', class: 'btn btn-danger', :method => :delete do
        content_tag(:i, nil, class: 'glyphicon glyphicon-remove')
      end  }"
    end
    "<div class='btn-group btn-group-nowrap-#{_wrp} btn-group-xs  pull-right' >
         #{update_button}
    #{read_button}
    #{delete_button}
      </div>".html_safe

  end

end