class Address < ActiveRecord::Base


  has_one  :institution, inverse_of: :address

  validates :street, :number, :city, :region, :zip, :country, presence: true

  alias_attribute :lat, :latitude
  alias_attribute :lng, :longitude
  alias_attribute :state, :region


  def real?
    result = _query_address_service
    return false if result == nil
    ok = result.is_precise? && !result.street.blank? && !result.number.blank? && !result.city.blank?
    ok &= !result.region.blank? && !result.zip.blank? && !result.country.blank?  && result.country == 'DE'
    ok
  end

  def complement!
    if real?
      result = _query_address_service
      self.street = result.street if self.street.blank?
      self.number = result.number if self.number.blank?
      self.city = result.city if self.city.blank?
      self.region = result.region if self.region.blank?
      self.zip = result.zip if self.zip.blank?
      self.country = result.country if self.country.blank?
      self.latitude= result.latitude if self.latitude.blank?
      self.longitude= result.longitude if self.longitude.blank?
      # this prevents the address service from being queried for an address it just returned as valid:
      @address_line_last = address_line
    end
  end

  def replace!
    if real?
      result = _query_address_service
      self.street = result.street
      self.number = result.number
      self.city = result.city
      self.region = result.region
      self.zip = result.zip
      self.country = result.country
      self.latitude= result.latitude
      self.longitude= result.longitude
      # this prevents the address service from being queried for an address it just returned as valid:
      @address_line_last = address_line
    end
  end



  def exists?
    return false unless  longitude.present? && latitude.present?
    return self.class.where(longitude: longitude, latitude: latitude ).present?
  end

  def address_line=(address_line)
    @address_line = address_line
  end

  def address_line
    @address_line = self.street.nil? ? '' : (self.street + ' ')
    @address_line += self.number.nil? ? '' : (self.number + ', ')
    @address_line += self.zip.nil? ? '' : (self.zip + ' ')
    @address_line += self.city.nil? ? '' : (self.city + ', ')
    @address_line += self.region.nil? ? '' : (self.region + ' ')
    @address_line += self.country.nil? ? '' : (self.country + ' ')
    @address_line
  end


  private
  def _query_address_service
    _address_line = address_line
    if @result.nil? || @address_line_last != _address_line
      @result = AddressGeocoder::ResultWrapperFactory.createFromSearchString(_address_line)
      @address_line_last = _address_line
    end
    @result
  end

end
