module Publishable
  extend ActiveSupport::Concern

  included do
    publish_callbacks.each do |name|
      send name, (Proc.new do |model|
        event = "#{model.class.name.underscore}.#{name}.active_record"
        Rails.logger.info "#{self.class.inspect}.#{name}.event #{event}"
        ActiveSupport::Notifications.instrument(event, model: model )
        true
      end)
    end
  end

  module ClassMethods
    def publish_callbacks
      %w(before_save after_save after_create after_update before_destroy after_destroy)
    end
  end

end