class Manager < ActiveRecord::Base
  
  #### Verwaltungsmitarbeiter: alle Personen, die administrative Aufgaben in einer Einrichtung wahrnehmen

  ENUM_GENDER_MALE = 0
  ENUM_GENDER_FEMALE = 1

  include Identifiable

  has_many :institution_parents, class_name: 'InstitutionHierarchy', foreign_key: :child_id

  #--- Associations
  belongs_to :institution

  #--- Attributes
  enum gender: { "männlich" => ENUM_GENDER_MALE, "weiblich" => ENUM_GENDER_FEMALE }

  #--- General Validations
  validates :first_name, :last_name, :gender, :institution_id, :presence => true
  validates :email, email: {}
  validates :phone, :mobile, :fax, phone: {}

  before_destroy :is_destructable?

  def name
    first_name + " " + last_name
  end

  def system_notifications
    ::Notification.only_showable.all
  end

  def is_destructable?
    true
  end
end
