module Notifications
    class Base
      include ::Singleton

      def self.available
        [
          ManagementConference, ParentTeacherConference, PortfolioRuleNotifications,
          ClientConfidentialityDocumentMissing, ClientClientGroupMissing, ClientCheckingForDuplicates, DiagnosticResultIncomplete,
          EducationalCourseViolations, BorrowAfterPortfolioEdit
          # TODO REMOVE: ClientInstitutionMissing (Schüler ohne Einrichtung muss durch Constraint verhindert werden)
          # TODO REMOVE: ClientGroupUpdateClients (Leere Klassen tun keinem Weh, spammen aber die Benachrichtigungen voll)
        ]
      end

      def self.subscribe
        available.each do |klass|
          klass.instance.subscribe
        end
      end


      # Example:
      # [{
      #   klass: "portfolio/items/*"
      #   callback: "after_destroy"
      #   handler: 'handler_method_1'
      # }]
      #
      # Means: any class in portfolio/items will cause handler_method_1 to be executed on after destroy
      #
      # Example:
      # [{
      #   klass: "portfolio/items/client"
      #   callback: "after_save"
      #   handler: 'handler_method_2'
      # }]
      #
      # Means: ::Portfolio::Items::Client will cause handler_method_2 to be executed on after save
      def subscribe_to
        []
      end

      def subscribe
        beeing_executed ={}
        subscribe_to.each do |options|
          if options[:klass].include?('*') # we do not support full regex, only * for a glob like behavior
            event = Regexp.new("#{options[:klass].gsub(/\*/, '.*?')}#{Regexp.escape(".#{options[:callback]}.active_record")}")
          else
            event = "#{options[:klass]}.#{options[:callback]}.active_record"
          end
          Rails.logger.info "#{self.class.inspect}.#{__method__}.event #{event}"
          handler_method = options[:handler]
          Rails.logger.info "#{self.class.inspect}.#{__method__}.handler_method #{handler_method}"
          ::ActiveSupport::Notifications.subscribe(event) do |name, start, finish, id, payload|
            unless beeing_executed[handler_method]
              beeing_executed[handler_method] = true
              begin
                Rails.logger.info "#{self.class.inspect}.notification_handler #{handler_method} #{ payload[:model].class.name}}"
                send handler_method, payload[:model]
              rescue ArgumentError => e
                Rails.logger.error "#{self.class.inspect}.notification_handler #{handler_method} #{e}"
              end
              beeing_executed[handler_method] = false
            end
          end
        end
      end

    end
end