module Notifications
  class BorrowAfterPortfolioEdit < Base

    def subscribe_to
      [
          {
              klass: 'portfolio/item_diagnostic',
              callback: 'after_create',
              handler: 'portfolio_item_diagnostic_after_create'
          },
          {
              klass: 'portfolio/item_advancement_plan',
              callback: 'after_create',
              handler: 'portfolio_item_advancement_plan_after_create'
          }
      ]
    end

    def create_diagnostic_notification(portfolio_item_diagnostic)
      portfolio_item = portfolio_item_diagnostic.portfolio_item
      return Notification.new({
          content: "#{portfolio_item.portfolio_step.name} für #{portfolio_item_diagnostic.portfolio_subject.name} hat neue Erhebungsmethode: #{portfolio_item_diagnostic.diagnostic.title}. Ausleihen?",
          notificationable: portfolio_item,
          secondary_notificationable: portfolio_item_diagnostic.diagnostic, # this is only for the resolve path
          institution_id: portfolio_item.portfolio_subject.institution_id,
          client_id:  ((portfolio_item.portfolio.subject.is_a? ::Client) ? portfolio_item.portfolio.subject.id : nil),
          start_at: DateTime.now,
          due_at: portfolio_item_diagnostic.begin,
          prio: 0,
          identifier: Notification::IDENTIFIER_BORROW_PORTFOLIO_DIAGNOSTIC
      })
    end

    def create_advancement_plan_notification(portfolio_item_advancement_plan)
      portfolio_item = portfolio_item_advancement_plan.portfolio_item
      return Notification.new({
          content: "#{portfolio_item.portfolio_step.name} für #{portfolio_item_advancement_plan.portfolio_subject.name} hat neue Fördermethode: #{portfolio_item_advancement_plan.advancement_plan.title}. Ausleihen?",
          notificationable: portfolio_item,
          secondary_notificationable: portfolio_item_advancement_plan.advancement_plan,  # this is only for the resolve path
          institution_id: portfolio_item.portfolio_subject.institution_id,
          client_id:  ((portfolio_item.portfolio.subject.is_a? ::Client) ? portfolio_item.portfolio.subject.id : nil),
          start_at: DateTime.now,
          due_at: portfolio_item_advancement_plan.begin,
          prio: 0,
          identifier: Notification::IDENTIFIER_BORROW_PORTFOLIO_ADVANCEMENT_PLAN
      })
    end

    def portfolio_item_diagnostic_after_create(portfolio_item_diagnostic)
      portfolio_item = portfolio_item_diagnostic.portfolio_item
      portfolio_item.notifications.where(identifier: Notification::IDENTIFIER_BORROW_PORTFOLIO_DIAGNOSTIC).destroy_all # we only keep the most recent
      create_diagnostic_notification(portfolio_item_diagnostic).save(validate: false)
    end

    def portfolio_item_advancement_plan_after_create(portfolio_item_advancement_plan)
      portfolio_item = portfolio_item_advancement_plan.portfolio_item
      portfolio_item.notifications.where(identifier: Notification::IDENTIFIER_BORROW_PORTFOLIO_ADVANCEMENT_PLAN).destroy_all # we only keep the most recent
      create_advancement_plan_notification(portfolio_item_advancement_plan).save(validate: false)
    end


  end
end