module Notifications
  class ClientCheckingForDuplicates < Base


    def subscribe_to
      [
          {
              klass: 'client_duplicate_request',
              callback: 'after_create',
              handler: 'client_duplicate_request_after_create'
          },
          {
              klass: 'client_duplicate_request',
              callback: 'before_save',
              handler: 'client_duplicate_request_before_save'
          },
          {
              klass: 'client_duplicate_request',
              callback: 'before_destroy',
              handler: 'client_duplicate_request_before_destroy'
          },
          {
              klass: 'client',
              callback: 'before_save',
              handler: 'client_before_save'
          },
          {
              klass: 'client',
              callback: 'after_create',
              handler: 'client_after_create'
          },
      ]
    end

    def create_client_duplicate_request_create_notification(client_duplicate_request)
      return Notification.new({
          content: ::I18n.translate('system_notifications.client_checking_for_duplicates.content',
                                    client_name: client_duplicate_request.first_name + " " + client_duplicate_request.last_name,
          ),
          notificationable: client_duplicate_request,
          institution_id: client_duplicate_request.requesting_institution.id,
          client_id: nil, # the client does not exist atm
          start_at: DateTime.now(),
          due_at: DateTime.now() + 10.seconds,
          identifier: Notification::IDENTIFIER_CLIENT_DUPLICATE_CREATE_REQUEST,
          prio: 1
      })
    end
    private :create_client_duplicate_request_create_notification


    def create_client_duplicate_request_duplicate_notification(client_duplicate_request)
      return Notification.new({
          content: ::I18n.translate('system_notifications.client_duplicate_result_received.content',
                                    client_name: client_duplicate_request.first_name + " " + client_duplicate_request.last_name,
          ),
          notificationable: client_duplicate_request,
          institution_id: client_duplicate_request.requesting_institution.id,
          client_id: nil, # the client does not exist atm
          start_at: DateTime.now(),
          due_at: DateTime.now(),
          identifier: Notification::IDENTIFIER_CLIENT_DUPLICATE_DUPLICATE_REQUEST,
          prio: 1
      })
    end
    private :create_client_duplicate_request_duplicate_notification

    def create_client_duplicate_request_client_created_notification(client)
      return Notification.new({
          content: ::I18n.translate('system_notifications.client_duplicate_client_created.content',
                                    client_name: client.first_name + " " + client.last_name,
          ),
          notificationable: client,
          institution_id: client.institution.id,
          client_id: client.id,
          start_at: DateTime.now(),
          due_at: nil, # this is just informative, no escalation
          identifier: Notification::IDENTIFIER_CLIENT_DUPLICATE_REQUEST_CLIENT_CREATED,
          prio: 0
      })
    end
    private :create_client_duplicate_request_client_created_notification

    # A request has been inserted and will now be processed
    def client_duplicate_request_after_create(client_duplicate_request)
      ::Notification.where('notificationable_type=? AND notificationable_id=?','ClientDuplicateRequest',client_duplicate_request.id).delete_all
      create_client_duplicate_request_create_notification(client_duplicate_request).save(validate:false)
    end

    def client_duplicate_request_before_save(client_duplicate_request)
      if client_duplicate_request.result_code_changed? && client_duplicate_request.request_finished?
        if client_duplicate_request.has_any_duplicates?
          ::Notification.where('notificationable_type=? AND notificationable_id=?','ClientDuplicateRequest',client_duplicate_request.id).delete_all
          create_client_duplicate_request_duplicate_notification(client_duplicate_request).save(validate:false)
        else

        end
      end
    end

    def client_duplicate_request_before_destroy(client_duplicate_request)
      ::Notification.where('notificationable_type=? AND notificationable_id=?','ClientDuplicateRequest',client_duplicate_request.id).delete_all
    end

    def client_before_save(client)
      if client.new_record? && client.client_duplicate_request.present?
        ::Notification.where('notificationable_type=? AND notificationable_id=?','ClientDuplicateRequest',client.client_duplicate_request_id).delete_all
      end
    end

    def client_after_create(client)
      create_client_duplicate_request_client_created_notification(client).save(validate:false)
    end

  end
end