module Notifications
  class ClientClientGroupMissing < Base


    def subscribe_to
      [
          {
              klass: 'system_setting',
              callback: 'after_save',
              handler: 'system_settings_after_save'
          },
          {
              klass: 'client',
              callback: 'after_save',
              handler: 'clients_after_save'
          },
      ]
    end

    def create_end_year_client_group_notification(semester, client)
      return Notification.new({
          content: ::I18n.translate('system_notifications.client_client_group_missing.content',
                                    client_name: client.first_name + " " + client.last_name
          ),
          notificationable: client,
          institution_id: client.institution_id,
          client_id: client.id,
          start_at: semester.finish.ago(60 * 60 * 24 * 7 * 4), # show these notifications not earlier than 4 weeks before semester finish
          due_at: semester.finish,
          identifier: Notification::IDENTIFIER_CLIENT_GROUP_MISSING,
          prio: 2
      })
    end

    def system_settings_after_save(system_setting)
      # schuljahreswechsel findet statt?
      if system_setting.semester_id_changed?
        # delete all notifications for last semester (in fact: all existing )
        ::Notification.where('identifier=?',Notification::IDENTIFIER_CLIENT_GROUP_MISSING).delete_all
        Client.where(next_semester_client_group_id: nil).each do |client|
          # create a notification for followup client group for all clients that have none
          create_end_year_client_group_notification(system_setting.semester, client).save(validate:false)
        end
      end
    end

    def clients_after_save(client)
      if client.next_semester_client_group_id_changed?
        if client.next_semester_client_group_id.blank?
          # create a notification for the client
          create_end_year_client_group_notification(SystemSetting.get_instance.semester, client).save(validate:false)
        else
          ::Notification.where('identifier=?',Notification::IDENTIFIER_CLIENT_GROUP_MISSING).where('notificationable_type=? AND notificationable_id=?','Client',client.id).delete_all
        end
      end
    end


  end
end