module Notifications
  class ClientConfidentialityDocumentMissing < Base

    # Benachrichtigung wenn die Schweigepflichtserklärung für einen Schüler / eine Schülerin fehlt:
    # wird geprüft, wenn die Zuordnung zu einer externen Person (ClientSupporterRelation) gespeichert wird
    # - wenn die Schweigepflichtserklärung dann (weiterhin oder erneut) fehlt, wird die Benachrichtigung erzeugt
    # - ansonsten werden entsprechende Benachrichtigungen gelöscht

    def subscribe_to
      [
          {
              klass: 'client_supporter_relation',
              callback: 'after_save',
              handler: 'client_supporter_relation_after_save'
          }
      ]
    end

    def create_notification(client_supporter_relation)
      client = client_supporter_relation.client
      supporter = client_supporter_relation.supporter
      return Notification.new({
        content: ::I18n.translate('system_notifications.client_confidentiality_document_missing.content',
                             client_name: client.first_name + " " + client.last_name,
                             supporter_name: supporter.first_name + " " + supporter.last_name
            ),
        notificationable: client_supporter_relation,
        institution_id: client.institution_id,
        client_id: client.id,
        start_at: client_supporter_relation.updated_at,
        due_at: Time.now,
        identifier: Notification::IDENTIFIER_CLIENT_CONFIDENTIALITY_DOCUMENT_MISSING,
        prio: 2
      })
    end

    def client_supporter_relation_after_save(client_supporter_relation)
      unless client_supporter_relation.documents.where(category: Document::ENUM_CATEGORY_CONFIDENTIALITY_DOCUMENT).any?
        unless client_supporter_relation.notifications
          .where(identifier: Notification::IDENTIFIER_CLIENT_CONFIDENTIALITY_DOCUMENT_MISSING)
          .any?
          create_notification(client_supporter_relation).save
        end
      else
        client_supporter_relation.notifications
          .where(identifier: Notification::IDENTIFIER_CLIENT_CONFIDENTIALITY_DOCUMENT_MISSING)
          .each do |notification|
            notification.destroy
          end
      end
    end

  end
end