module Notifications
  class ClientGroupUpdateClients < Base

    # Benachrichtigung wenn einer Klasse des aktuellen Semesters
    # bisher keine SchülerInnen zugeordnet wurden:
    # wird geprüft, wenn eine Klasse gespeichert wird
    # - wenn (weiterhin oder erneut) keine SchülerInnen zugeordnet sind, wird die Benachrichtigung erzeugt
    # - ansonsten werden entsprechende Benachrichtigungen gelöscht
    # wird geprüft, wenn ein Client-Datensatzes gespeichert wird
    # - wenn eine Klasse zugeordnet ist, wird die Benachrichtigung für diese Klasse gelöscht
    # - wenn in der zuvor ausgewählten Klasse (weiterhin oder erneut) keine SchülerInnen zugeordnet sind, wird die Benachrichtigung erzeugt

    def subscribe_to
      [
          {
              klass: 'client_group',
              callback: 'after_save',
              handler: 'client_group_after_save'
          },
          {
              klass: 'client',
              callback: 'after_save',
              handler: 'client_after_save'
          }
      ]
    end

    def create_notification(client_group)
      return Notification.new({
          content: "In der Klasse " + client_group.name + " sind bisher keine SchülerInnen.",
          notificationable: client_group,
          institution_id: client_group.institution_id,
          start_at: client_group.created_at,
          due_at: Time.now,
          identifier: Notification::IDENTIFIER_CLIENT_GROUP_UPDATE_CLIENTS
      })
    end

    def client_group_after_save(client_group)
      if client_group.clients.blank? && ( SystemSetting.get_instance.semester_id == client_group.semester.id)
        unless client_group.notifications
          .where(identifier: Notification::IDENTIFIER_CLIENT_GROUP_UPDATE_CLIENTS)
          .any?
          create_notification(client_group).save
        end
      else
        client_group.notifications
          .where(identifier: Notification::IDENTIFIER_CLIENT_GROUP_UPDATE_CLIENTS)
          .delete_all
      end
    end

    def client_after_save(client)
      if previous_client_group_id = client.client_group_id_was
        if previous_client_group_id != client.client_group_id
          previous_client_group = ClientGroup.find(previous_client_group_id)
          client_group_after_save(previous_client_group)
        end
      end
      if client.client_group.present?
        client.client_group.notifications
          .where(identifier: Notification::IDENTIFIER_CLIENT_GROUP_UPDATE_CLIENTS)
          .delete_all
      end
    end



  end
end