module Notifications
  class ClientInstitutionMissing < Base

    # Benachrichtigung wenn die Einrichtungs-Zugehörigkeit für einen Schüler / eine Schülerin fehlt:
    # wird geprüft, wenn ein Client-Datensatzes gespeichert wird
    # - wenn die Einrichtungs-Zugehörigkeit dann (weiterhin oder erneut) fehlt, wird die Benachrichtigung erzeugt
    # - ansonsten werden entsprechende Benachrichtigungen gelöscht

    def subscribe_to
      [
          {
              klass: 'client',
              callback: 'after_save',
              handler: 'client_after_save'
          }
      ]
    end

    def create_notification(client)
      return Notification.new({
          content: client.first_name + " " + client.last_name + " ist keiner Einrichtung zugehörig.",
          notificationable: client,
          institution_id: client.institution_id,
          client_id: client.id,
          start_at: Time.now,
          due_at: Time.now,
          identifier: Notification::IDENTIFIER_CLIENT_INSTITUTION_MISSING,
          prio: 2
      })
    end

    def client_after_save(client)
      if client.institution_id_changed? || client.id_changed?
        if client.institution_id.nil?
          create_notification(client).save
        else
          client.notifications
            .where(identifier: Notification::IDENTIFIER_CLIENT_INSTITUTION_MISSING)
            .each do |notification|
              notification.destroy
          end
        end
      end
    end

  end
end