module Notifications
  class DiagnosticResultIncomplete < Base
    
      def subscribe_to
        [
            {
                klass: 'portfolio/item',
                callback: 'before_save',
                handler: 'portfolio_item_before_save'
            },
            {
                klass: 'portfolio/item',
                callback: 'after_destroy',
                handler: 'portfolio_item_after_destroy'
            }
        ]
      end

      # @param portfolio_item ::Portfolio::Item
      def create_notification(portfolio_item)
        notification = ::Notification.new
        notification.notificationable= portfolio_item
        notification.institution_id= portfolio_item.portfolio_subject.institution_id
        notification.client_id = portfolio_item.portfolio.subject.id if portfolio_item.portfolio.subject.is_a? ::Client
        notification.content= ::I18n.translate('system_notifications.diagnostic_result_incomplete.content',
                                               step: portfolio_item.portfolio_step.name,
                                               step_date: ::I18n.localize(portfolio_item.start,format: :short)
        )
        notification.start_at=portfolio_item.start
        notification.due_at= portfolio_item.start
        notification.identifier= Notification::IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE
        notification.prio= 0
        notification
      end

      # jedes item, das eine parent_teacher_conference verlangt, erzeugt genau eine benachrichtigung.
      # Eine parent_teacher_conference löscht alle davor erzeugten Benachrichtigungen
      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_before_save(portfolio_item)
        all_results_closed = true
        portfolio_item.diagnostics.each do |diagnostic|
          all_results_closed &= diagnostic.results_closed
        end
        portfolio_item.diagnostic_measurements.each do |diagnostic|
          all_results_closed &= diagnostic.results_closed
        end
        portfolio_item.class_diagnostics.each do |diagnostic|
          all_results_closed &= diagnostic.results_closed
        end
        if all_results_closed
          ::Notification.where('identifier=?',Notification::IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).each do |notification|
            # wenn ja, lösche die benachrichtigung für genau dieses Item
            notification.destroy
          end
        else
          unless ::Notification.where('identifier=?',Notification::IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).exists?
            # wenn nein, erstelle eine benachrichtigung für genau dieses Item
            create_notification(portfolio_item).save(validate: false)
          end
        end
      end

      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_after_destroy(portfolio_item)
        ::Notification.where('identifier=?',Notification::IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).each do |notification|
          # wenn ja, lösche die benachrichtigung für genau dieses Item
          notification.destroy
        end
      end

    end
end