module Notifications
  class ManagementConference < Base

      # list of model_name.callback
      def subscribe_to
        [
            {
                klass: 'portfolio/item',
                callback: 'before_save',
                handler: 'portfolio_item_before_save'
            },
            {
                klass: 'portfolio/item',
                callback: 'after_destroy',
                handler: 'portfolio_item_after_destroy'
            }
        ]
      end

      def create_notification(portfolio_item)
        notification = ::Notification.new
        notification.notificationable= portfolio_item
        notification.institution_id= portfolio_item.portfolio_subject.institution_id
        notification.client_id = portfolio_item.portfolio.subject.id if portfolio_item.portfolio.subject.is_a? ::Client
        notification.content= ::I18n.translate('system_notifications.management_conference.content',
                                               step: portfolio_item.portfolio_step.name,
                                               step_date: ::I18n.localize(portfolio_item.start,format: :short),
                                               management_conference_date: ::I18n.localize(portfolio_item.management_conference_date,format: :short),
        )

        notification.start_at= Date.today()
        notification.due_at= portfolio_item.management_conference_date
        notification.identifier= Notification::IDENTIFIER_MANAGEMENT_CONFERENCE
        notification.prio= 0
        notification
      end

      # jedes item, das eine management_conference verlangt, erzeugt genau eine benachrichtigung.
      # Eine management_conference löscht alle davor erzeugten Benachrichtigungen
      def portfolio_item_before_save(portfolio_item)
        if portfolio_item.management_conference_changed?
          if portfolio_item.management_conference?
            # es gibt später eine management conference?
            unless portfolio_item.portfolio.portfolio_items.includes(:portfolio_step).where('start >= ?',portfolio_item.start).where('portfolio_steps.type= ?','management_conference').references(:portfolio_step).exists?
              unless ::Notification.where('identifier=?',Notification::IDENTIFIER_MANAGEMENT_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).exists?
               create_notification(portfolio_item).save(validate: false)
              end
            end
          else
            ::Notification.where('identifier=?',Notification::IDENTIFIER_MANAGEMENT_CONFERENCE).where('notificationable_type=?','Portfolio::Item').where('notificationable_id=?',portfolio_item.id).each do |notification|
              notification.destroy
            end
          end
        end
        if portfolio_item.portfolio_step.type == 'management_conference'
          ::Portfolio::Item.where('start <= ?',portfolio_item.start).joins(:portfolio).where('portfolios.subject_id = ? AND portfolios.subject_type=?', portfolio_item.portfolio.subject_id,portfolio_item.portfolio.subject_type).where('management_conference= ?',true).each do |older_item|
            ::Notification.where('identifier=?',Notification::IDENTIFIER_MANAGEMENT_CONFERENCE).where('notificationable_type=?','Portfolio::Item').where('notificationable_id=?',older_item.id).each do |notification|
              notification.destroy
            end
          end
        end
      end

      def portfolio_item_after_destroy(portfolio_item)
        if portfolio_item.portfolio_step.allowed?(:management_conference)
          if portfolio_item.management_conference?
            ::Notification.where('identifier=?',Notification::IDENTIFIER_MANAGEMENT_CONFERENCE).where('notificationable_type=?','Portfolio::Item').where('notificationable_id=?',portfolio_item.id).each do |notification|
              notification.destroy
            end
          end
        end
        if portfolio_item.portfolio_step.type == 'management_conference'
          if older_item =  ::Portfolio::Item.where('start <= ?',portfolio_item.start).where('portfolio_id = ?',portfolio_item.portfolio_id).where('management_conference= ?',true).take
            unless ::Notification.where('identifier=?',Notification::IDENTIFIER_MANAGEMENT_CONFERENCE).where('notificationable_type=?','Portfolio::Item').where('notificationable_id=?',older_item.id).exists?
              create_notification(portfolio_item).save(validate: false)
            end
          end
        end
      end

    end
end