module Notifications
  class ParentTeacherConference < Base
    
      def subscribe_to
        [
            {
                klass: 'portfolio/item',
                callback: 'before_save',
                handler: 'portfolio_item_before_save'
            },
            {
                klass: 'portfolio/item',
                callback: 'after_destroy',
                handler: 'portfolio_item_after_destroy'
            }
        ]
      end

      # @param portfolio_item ::Portfolio::Item
      def create_notification(portfolio_item)
        notification = ::Notification.new
        notification.notificationable= portfolio_item
        notification.institution_id= portfolio_item.portfolio_subject.institution_id
        notification.client_id = portfolio_item.portfolio.subject.id if portfolio_item.portfolio.subject.is_a? ::Client
        notification.content= ::I18n.translate('system_notifications.parent_teacher_conference.content',
                                               step: portfolio_item.portfolio_step.name,
                                               step_date: ::I18n.localize(portfolio_item.start,format: :short),
                                               management_conference_date: ::I18n.localize(portfolio_item.parent_teacher_conference_date,format: :short),
        )

        notification.start_at= Date.today()
        notification.due_at= portfolio_item.parent_teacher_conference_date
        notification.identifier= Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE
        notification.prio= 0
        notification
      end

      # jedes item, das eine parent_teacher_conference verlangt, erzeugt genau eine benachrichtigung.
      # Eine parent_teacher_conference löscht alle davor erzeugten Benachrichtigungen
      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_before_save(portfolio_item)
        if portfolio_item.parent_teacher_conference_changed?
          # verlangt es eine parent teacher konference?
          if portfolio_item.parent_teacher_conference?
            # prüfe ob eine danach folgt.
            unless portfolio_item.portfolio.portfolio_items.includes(:portfolio_step).where('start >= ?',portfolio_item.start).where('portfolio_steps.type= ?','parent_teacher_conference').references(:portfolio_step).exists?
              # wenn nein, prüfe, ob schon eine benachrichtigung für genau dieses Item existiert?
              unless ::Notification.where('identifier=?',Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).exists?
                # wenn nein, erstelle eine benachrichtigung für genau dieses Item
               create_notification(portfolio_item).save(validate: false)
              end
            end
          else
            # das item verlangt jetzt keine konferenz mehr
            # prüfe, ob schon eine benachrichtigung für genau dieses Item existiert?
            ::Notification.where('identifier=?',Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).each do |notification|
              # wenn ja, lösche die benachrichtigung für genau dieses Item
              notification.destroy
            end
          end
        end
        # oder ist es eine parent teacher conference?
        if portfolio_item.portfolio_step.type == 'parent_teacher_conference'
          # prüfe ob davor eine verlangt wurde
          portfolio_item.portfolio.portfolio_items.where('start <= ?',portfolio_item.start).where('parent_teacher_conference= ?',true).each do |older_item|
            # wenn ja, prüfe ob es eine benachrichtigung gibt,
            ::Notification.where('identifier=?',Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',older_item.id).each do |notification|
              # wenn ja, lösche sie
              notification.destroy
            end
          end
        end
      end

      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_after_destroy(portfolio_item)
        if portfolio_item.portfolio_step.allowed?(:parent_teacher_conference)
          if portfolio_item.parent_teacher_conference?
            # gibt es einen benachrichtigung, die von diesem Item erzeugt wurde?
            ::Notification.where('identifier=?',Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',portfolio_item.id).each do |notification|
              # wenn ja, lösche sie
              notification.destroy
            end
          end
        end
        # oder ist es eine parent teacher conference?
        if portfolio_item.portfolio_step.type == 'parent_teacher_conference'
          # prüfe ob davor eine verlangt wurde
          if older_item =  ::Portfolio::Item.where('start <= ?',portfolio_item.start).where('portfolio_id = ?',portfolio_item.portfolio_id).where('parent_teacher_conference= ?',true).take
            # wenn ja, prüfe ob es eine benachrichtigung gibt,
            unless ::Notification.where('identifier=?',Notification::IDENTIFIER_PARENT_TEACHER_CONFERENCE).where('notificationable_type=?',portfolio_item.class.name).where('notificationable_id=?',older_item.id).exists?
              # wenn nein, erzeuge sie
              create_notification(portfolio_item).save(validate: false)
            end
          end
        end
      end

    end
end