module Notifications
  class PortfolioRuleNotifications < Base

      def subscribe_to
        [
            {
                klass: 'portfolio/item',
                callback: 'after_save',
                handler: 'portfolio_item_after_save'
            },
            {
                klass: 'portfolio/item',
                callback: 'after_destroy',
                handler: 'portfolio_item_after_destroy'
            },
            {
                klass: 'career_item',
                callback: 'after_create',
                handler: 'career_item_after_create'
            }
        ]
      end

      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_after_save(portfolio_item)
        if portfolio_item.closed?
          apply_rule_notifications(portfolio_item.portfolio)
        end
      end


      # @param portfolio_item ::Portfolio::Item
      def portfolio_item_after_destroy(portfolio_item)
        if portfolio_item.closed?
          apply_rule_notifications(portfolio_item.portfolio)
        end
      end

      # @param career_item ::CareerItem
      def career_item_after_create(career_item)
        return unless career_item[:change_type] == CareerItem::CHANGE_TYPE_LEVEL
        apply_rule_notifications(career_item.client.running_portfolio)
      end

      private
      def apply_rule_notifications(portfolio)
        if portfolio.next_rule_notifiers.blank?
          ::Notification
            .where('notificationable_type=?',portfolio.class.name)
            .where('notificationable_id=?',portfolio.id)
            .where("identifier LIKE ?", "#{::Notification::PREFIX_IDENTIFIER_PORTFOLIO_RULE}/%")
          .each do |notification|
            notification.destroy
          end
        else
          ::Notification
              .where("identifier NOT LIKE ?", "#{::Notification::PREFIX_IDENTIFIER_PORTFOLIO_RULE}/#{portfolio.next_rule_position}/%")
              .where("identifier LIKE ?", "#{::Notification::PREFIX_IDENTIFIER_PORTFOLIO_RULE}/%")
              .where('notificationable_type=?',portfolio.class.name)
              .where('notificationable_id=?',portfolio.id)
          .each do |notification|
            notification.destroy
          end
          portfolio.next_rule_notifiers.each do |notifier|
            if notifier.show?
              notifier.create
            else
              notifier.destroy
            end
          end
        end
        return true
      end

    end
end