module Portfolio
  class ItemEvaluationDbr < ::ActiveRecord::Base
    self.table_name = 'portfolio_item_evaluation_dbrs'
    self.inheritance_column = nil
    # t.float :range_begin
    # t.float :range_end
    # t.string :setting_social
    # t.string :setting_time
    # t.integer :number_of_data_points


    has_one   :portfolio_item_evaluation,
               foreign_key: :portfolio_item_evaluation_dbr_id,
               class_name: '::Portfolio::ItemEvaluation',
               inverse_of:  :portfolio_item_evaluation_dbr
    delegate :portfolio_item, to: :portfolio_item_evaluation
    delegate :portfolio, to: :portfolio_item
    delegate :portfolio_subject, to: :portfolio_item

    has_many  :characteristics,
                class_name: '::Portfolio::ItemEvaluationDbrCharacteristic',
                foreign_key: :portfolio_item_evaluation_dbr_id,
                inverse_of:  :dbr,
                dependent: :destroy
    accepts_nested_attributes_for :characteristics, reject_if: :all_blank

    has_many  :results,
              class_name: '::Portfolio::ItemEvaluationDbrResult',
              foreign_key: :portfolio_item_evaluation_dbr_id,
              inverse_of:  :dbr,
              dependent: :destroy
    accepts_nested_attributes_for :results, reject_if: :all_blank

    validates :range_begin, :range_end,:setting_social,:setting_time,:number_of_data_points, presence: true
    validates :range_begin, :range_end , numericality: true
    validates :setting_social, :setting_time , length: { maximum: 140, minimum: 10}
    validates :number_of_data_points , numericality: { only_integer: true }
    validate do
      errors.add(:characteristics, "Maximal 5 Merkmale können erfasst werden.") if (characteristics.size > 5)
      errors.add(:characteristics, "Mindestend 1 Merkmal sollte erfasst werden.") if (characteristics.size < 1)
    end

  end
end


