module Portfolio
  class ItemTargetResult < ::ActiveRecord::Base
    self.table_name = 'portfolio_item_target_results'
    self.inheritance_column = nil


    belongs_to :portfolio_item_target,
               class_name: '::Portfolio::ItemTarget',
               foreign_key: :portfolio_item_target_id,
               inverse_of: :results

    belongs_to :evaluative,
               foreign_key: :evaluative_id,
               polymorphic: true

    before_save :update_is_successful

    def self.result_schemes
      # Direct behavior rating is not yet ready, so exclude this result set
      # ["::Portfolio::ItemTargetResult::Binary","::Portfolio::ItemTargetResult::Quinary","::Portfolio::ItemTargetResult::Dbr"]
      ["::Portfolio::ItemTargetResult::Binary","::Portfolio::ItemTargetResult::Quinary"]
    end

    def result= result_scheme_obj
      self.result_scheme=result_scheme_obj.class
    end

    def result
      result_scheme_obj = self.result_scheme.constantize.new
      return result_scheme_obj
    end

    def result_string
      result.print(result_value)
    end

    def result_normalized
      result.normalize_value(result_value)
    end

    private
    def update_is_successful
      self.is_successful = result.is_successful(result_value)
      true
    end

    class Binary
      def self.label
        "2 stufig ( + / - )"
      end

      def range
        ["+","-"]
      end

      # gets value normalized around 0
      def normalize_value(value)
        return -1 if value == 1
        return 1 if value == 0
      end

      def input_partial
        'portfolio_item_targets/results/input/binary'
      end

      def print(value)
        return " -kein Ergebnis- " if value.nil?
        case range[value]
          when "+"
            return "<i class='glyphicon glyphicon-plus-sign'></i>".html_safe
          when "-"
            return "<i class='glyphicon glyphicon-minus-sign'></i>".html_safe
        end
      end

      def is_successful(value)
        return false if value.nil?
        value == 0
      end
    end
    class Dbr < Binary # this is like binary result in the end, but prior to the end result, there are lots of datapoints gathered, graphs drawn and the like...
      def self.label
        "Verlaufserfassung"
      end
    end
    class Quinary

      def self.label
        "5 stufig ( + + / + / o / - / - -)"
      end

      def range
        ["++","+","o","-","--"]
      end

      # gets value normalized around 0
      def normalize_value(value)
        return 2 if value == 0
        return 1 if value == 1
        return 0 if value == 2
        return -1 if value == 3
        return -2 if value == 4
      end

      def input_partial
        'portfolio_item_targets/results/input/quinary'
      end

      def print(value)
        return " -kein Ergebnis- " if value.nil?
        case range[value]
          when "++"
            return "<i class='glyphicon glyphicon-plus-sign'></i><i class='glyphicon glyphicon-plus-sign'></i>".html_safe
          when "+"
            return "<i class='glyphicon glyphicon-plus-sign'></i>".html_safe
          when "o"
            return "<i class='glyphicon glyphicon-record'></i>".html_safe
          when "-"
            return "<i class='glyphicon glyphicon-minus-sign'></i>".html_safe
          when "--"
            return "<i class='glyphicon glyphicon-minus-sign'></i><i class='glyphicon glyphicon-minus-sign'></i>".html_safe
        end
      end

      def is_successful(value)
        return false if value.nil?
        value <= 2
      end

    end

  end
end


