module Portfolio
  class Portfolio < ::ActiveRecord::Base
    self.table_name = 'portfolios'
    self.inheritance_column = nil

    include ::Notificationable

    has_many :portfolio_items,
             foreign_key: :portfolio_id,
             class_name: '::Portfolio::Item',
             inverse_of:  :portfolio,
             dependent: :destroy


    belongs_to :subject,
               polymorphic: true

    belongs_to :client, ->(portfolio) {  where(portfolio.subject_type=='Client' ? '1=1' : '1=0' )}, foreign_key: :subject_id, class_name: '::Client'

    belongs_to :client_group, ->(portfolio) {  where(portfolio.subject_type=='ClientGroup' ? '1=1' : '1=0' )}, foreign_key: :subject_id, class_name: '::ClientGroup'


    def most_recently_closed_item
      portfolio_items.most_recently_closed.take
    end

    # Provides a wrapper class for owners with a common interface
    # @return [::Portfolio::Subjects::Base]
    def portfolio_subject
      @portfolio_subject = "::Portfolio::Subjects::#{subject.class.name}".constantize.new(self,subject) unless @portfolio_subject.present?
      return @portfolio_subject
    end

    def allowed_steps
      chain.allowed_steps(portfolio_level)
    end

    def next_rule_description
      return "Alle Förderinstrumente sind verfügbar, eine Erweiterung ist nicht mehr möglich." unless  has_next_rule?
      return "Bevor neue Schritte verfügbar werden muss der Schüler in die nächste Förderstufe." if next_rule_level > portfolio_level
      chain.next_rule.description
    end

    def next_rule_level
      chain.next_level
    end

    def last_rule_level
      chain.last_level
    end

    def next_rule_notifiers
      return [] if chain.next_rule.nil?
      chain.next_rule.all_notifiers(self)
    end

    def next_rule_position
      return nil if chain.next_rule.nil?
      chain.next_rule.position
    end

    def next_rule_has_condition? name
      return false unless chain.has_next_rule?
      chain.next_rule.portfolio_conditions.each do |c|
        return true if c.has_matcher?(name)
      end
      return false
    end

    def block_level_raise?
      chain.block_level_raise?
    end

    def has_next_rule?
      chain.has_next_rule?
    end

    def last_matching_item
      chain.last_matching_item
    end

    def can_lower_portfolio_level(portfolio_level)
      last_rule_level < portfolio_level
    end

    def can_raise_portfolio_level(portfolio_level)
      portfolio_level < next_rule_level && !block_level_raise?
    end

    def allowed_levels
      [(has_last_rule? ? last_rule_level : 0),(has_next_rule? ? next_rule_level : 0)]
    end

    def portfolio_rules
      ::Portfolio::Rule.where(portfolio_rule_set_id: rule_set_id)
    end

    delegate :documents, to: :chain

    private
    def chain
      @chain = ::Portfolio::Chain.new(portfolio_items.where(closed: true),portfolio_rules,portfolio_level) if @chain.nil?
      @chain.run
      return @chain
    end



  end
end