class TeacherInstitutionRelation < ActiveRecord::Base

  #### Beziehung zwischen einer Lehrperson und einer Einrichtung

  #--- Associations
  belongs_to :teacher, inverse_of: :teacher_institution_relations
  has_many :clients, inverse_of: :teacher_institution_relations
  belongs_to :institution
  belongs_to :teacher_institution_category
  belongs_to :portfolio_specialization

  #--- General Validations
  validates :teacher,:institution,  presence: true
  validates :normal_education_part_time, presence: {
      message: "Die ausgewählte Einrichtung ist eine Lehreinrichtung. Bitte geben Sie einen Wochenstundenwert ein."
  }, if: :school_or_educational?
  validates :normal_education_part_time, absence: {
      message: "An der ausgewählten Einrichtung findet keine Lehre statt. Bitte lassen Sie das Feld leer."
  }, if: :organisational?
  validates :normal_education_part_time, numericality: true, if: :school_or_educational?

  validates :special_education_part_time, presence: {
      message: "Die ausgewählte Einrichtung ist eine Lehreinrichtung. Bitte geben Sie einen Wochenstundenwert ein."
  }, if: :special_ed_and_school?
  validates :special_education_part_time, numericality: true, if: :special_ed_and_school?

  before_destroy do
    # feature #157 klassenlehrer werden auf null gesetzt.
    ClientGroup.where(teacher_id: teacher_id, institution_id: institution_id).update_all(teacher_id: nil)
  end

  private
  def school_or_educational?
    return false if institution.blank?
    institution.institution_type == Institution::TYPE_SCHOOL ||
    institution.institution_type == Institution::TYPE_EDUCATIONAL
  end

  def organisational?
    return false if institution.blank?
    institution.institution_type == Institution::TYPE_ORGANISATIONAL
  end

  def special_ed_and_school?
    return false if teacher.blank?
    return false if teacher.teacher_category.blank?
    teacher.teacher_category.is_special_ed? && school_or_educational?
  end

end
