module CsbApi
  class AdvancementPlan < CsbApi::Base
    def self.table_name; "advancement-plans"; end

    # populate AdvancementPlan activerecord instance from this result
    #
    # When any relationships of type (advancement-plan-images)
    # are included (@see #JsonApiClient::Resource.includes) in this result,
    # the corresponding associations on AdvancementPlan are populated as well.
    #
    # When AdvancementPlan instance is omitted, a new instance is returned
    # When AdvancementPlan instance is set, it is populated and returned
    #
    # @param advancement_plan [AdvancementPlan] the instance to populate
    # @return [AdvancementPlan] the populated instance
    def build_model(advancement_plan = ::AdvancementPlan.new)
      advancement_plan.attributes={
          title: read_attribute('title'),
          abbreviation: read_attribute('abbreviation'),
          catalog_id: read_attribute('id'),
          authors: read_attribute('authors') ,
          year: read_attribute('year'),
          subtitle: read_attribute('subtitle') ,
          min_level: read_attribute('min-level') ,
          max_level: read_attribute('max-level') ,
          age_from: read_attribute('age-from'),
          age_to: read_attribute('age-to'),
          no_image_url: meta['no_image_url'],
          short_description: read_attribute('short-description'),
          description: read_attribute('description') ,
          url: read_attribute('url')
      }
      if relationships['advancement-plan-images'].key? "data" # the records are included in the result set
        remotes = self.send('advancement-plan-images').index_by(&:name)
        advancement_plan.advancement_plan_imgs.each do |img| # update
          if remotes.key? img.name
            remotes[img.name].populate_model(img)
            remotes.delete img.name
          end
        end
        remotes.each_value do |resource|  # create
          advancement_plan.advancement_plan_imgs << resource.build_model
        end

      end
      return advancement_plan
    end
    alias :populate_model :build_model


    class AdvancementPlanImage < CsbApi::Base
      def self.table_name; "advancement-plan-images"; end


      # populate AdvancementPlanImg activerecord instance from this result
      #
      # @param image [AdvancementPlanImg] the instance to populate
      # @return [AdvancementPlanImg] the populated instance
      def build_model(image = ::AdvancementPlanImg.new)
        image.attributes = {
            name: read_attribute('name'),
            width_px: read_attribute('width-px'),
            height_px: read_attribute('height-px'),
            source_url: meta['url'],
            file_type: read_attribute('file-type')
        }
        return image
      end

      alias :populate_model :build_model
    end

  end
end
