class ReloadPortfolioRules < ActiveRecord::Migration
  def up
    execute <<-SQL
       -- DROP AUTO INC SEQUENCE FOR portfolio_conditions_steps PK
      ALTER TABLE portfolio_conditions_steps ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_conditions_steps_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_steps PK
      ALTER TABLE portfolio_steps ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_steps_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_step_results PK
      ALTER TABLE portfolio_step_results ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_step_results_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_rules PK
      ALTER TABLE portfolio_rules ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_rules_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_rule_steps PK
      ALTER TABLE portfolio_rule_steps ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_rule_steps_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_document_types PK
      ALTER TABLE portfolio_document_types ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_document_types_id_seq;


      -- DROP AUTO INC SEQUENCE FOR portfolio_conditions PK
      ALTER TABLE portfolio_conditions ALTER COLUMN id DROP DEFAULT;
      DROP SEQUENCE portfolio_conditions_id_seq;

      -- These tables are not in use any more
      DROP TABLE portfolio_conditions_step_results,portfolio_conditions_document_types
    SQL
    add_column :portfolio_conditions_steps, :portfolio_rule_set_id, :string

    # if the following fails, we really got a problem.
    if SystemSetting.vsb_registered?
        steps = []
        Portfolio::Step.all.each do |step|
           mapped_rule_set = step.portfolio_rule_set_id
           mapped_rule_set = 'bzhelUvbO' if step.portfolio_rule_set_id.include? 'bzhelUvbO'
           mapped_rule_set = 'k7EwSR3oY' if step.portfolio_rule_set_id.include? 'k7EwSR3oY'
           mapped_name = step.name
           mapped_name = 'Beratung' if step.name == 'Förderplan' && mapped_rule_set ==  'k7EwSR3oY'
           mapped_name = 'Fördermethode' if step.name == 'Förderplan' && mapped_rule_set ==  'bzhelUvbO'
           mapped_name = 'Erhebungsmethode' if step.name == 'Diagnostik' && mapped_rule_set ==  'bzhelUvbO'
           steps << [mapped_rule_set,mapped_name,step.id,false]
        end
        remove_foreign_key :portfolio_items, :portfolio_step # this one is reindexed
        remove_foreign_key :portfolio_items, :portfolio_step_result # this one is just set to NULL, this means loss of data (results), but as the system did not go live yet, there is no data anyway
        execute <<-SQL
          TRUNCATE TABLE portfolio_conditions_steps,portfolio_steps,portfolio_conditions,portfolio_step_results,portfolio_rules,portfolio_document_types,portfolio_rule_steps;
        SQL
        SystemSetting.get_instance.load_latest_portfolio_rules
        SystemSetting.get_instance.load_latest_client_group_portfolio_rules
        say_with_time "reindexing" do
          new_steps = []
          Portfolio::Step.all.each do |step|
            mapped_rule_set = step.portfolio_rule_set_id
            mapped_rule_set = 'bzhelUvbO' if step.portfolio_rule_set_id.include? 'bzhelUvbO'
            mapped_rule_set = 'k7EwSR3oY' if step.portfolio_rule_set_id.include? 'k7EwSR3oY'
            new_steps << [mapped_rule_set,step.name,step.id]
          end
          new_steps.each do |new_step|
            steps.each do |step|
              if step[0] == new_step[0] && (step[1] == new_step[1])
                execute "UPDATE portfolio_items SET portfolio_step_id='#{new_step[2]}' WHERE portfolio_step_id='#{step[2]}' "
              end
            end
          end
          true
        end
        add_foreign_key :portfolio_items, :portfolio_steps
        add_foreign_key :portfolio_items, :portfolio_step_results
    end
  end

  def down
    execute <<-SQL
      -- CREATING AUTO INC SEQUENCE FOR portfolio_steps PK
      CREATE SEQUENCE portfolio_steps_id_seq;
      ALTER SEQUENCE portfolio_steps_id_seq OWNED BY portfolio_steps.id;
      ALTER TABLE portfolio_steps ALTER COLUMN id SET DEFAULT nextval('portfolio_steps_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_step_results PK
      CREATE SEQUENCE portfolio_step_results_id_seq;
      ALTER SEQUENCE portfolio_step_results_id_seq OWNED BY portfolio_step_results.id;
      ALTER TABLE portfolio_step_results ALTER COLUMN id SET DEFAULT nextval('portfolio_step_results_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_rules PK
      CREATE SEQUENCE portfolio_rules_id_seq;
      ALTER SEQUENCE portfolio_rules_id_seq OWNED BY portfolio_rules.id;
      ALTER TABLE portfolio_rules ALTER COLUMN id SET DEFAULT nextval('portfolio_rules_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_rule_steps PK
      CREATE SEQUENCE portfolio_rule_steps_id_seq;
      ALTER SEQUENCE portfolio_rule_steps_id_seq OWNED BY portfolio_rule_steps.id;
      ALTER TABLE portfolio_rule_steps ALTER COLUMN id SET DEFAULT nextval('portfolio_rule_steps_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_document_types PK
      CREATE SEQUENCE portfolio_document_types_id_seq;
      ALTER SEQUENCE portfolio_document_types_id_seq OWNED BY portfolio_document_types.id;
      ALTER TABLE portfolio_document_types ALTER COLUMN id SET DEFAULT nextval('portfolio_document_types_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_conditions PK
      CREATE SEQUENCE portfolio_conditions_id_seq;
      ALTER SEQUENCE portfolio_conditions_id_seq OWNED BY portfolio_conditions.id;
      ALTER TABLE portfolio_conditions ALTER COLUMN id SET DEFAULT nextval('portfolio_conditions_id_seq'::regclass);

      -- CREATING AUTO INC SEQUENCE FOR portfolio_conditions_steps PK
      CREATE SEQUENCE portfolio_conditions_steps_id_seq;
      ALTER SEQUENCE portfolio_conditions_steps_id_seq OWNED BY portfolio_conditions_steps.id;
      ALTER TABLE portfolio_conditions_steps ALTER COLUMN id SET DEFAULT nextval('portfolio_conditions_steps_id_seq'::regclass);
    SQL
  end
end
