#!/bin/bash
#-----------------------------------------------------------------------------------------------------------------------
#
# This is the start script for the application server
# The server will start in foreground, it is adviseable to put this script into init / systemd
# so, a respawn / restart directive can be applied
#
# USAGE: srvctl.sh { start | stop | restart } { production | development }
#
#-----------------------------------------------------------------------------------------------------------------------

INSTANZ="Vschoolboard Application Server"
SCRIPT=$(readlink -f "$0")
SCRIPTPATH=$(dirname "$SCRIPT")
APP_ENVIRONMENT="$2"
cd $SCRIPTPATH
Pidfile=tmp/pids/puma.pid

if [ -f $Pidfile ]
then
	Pid=`cat $Pidfile`
fi

case "$1" in
'start')
		if [ -f $Pidfile ] ; then
				if test `ps -e | grep -c $Pid` = 1; then
						echo "Not starting $INSTANZ - instance already running with PID: $Pid"
				else
						echo "Starting $INSTANZ"
						bundle exec puma -C "config/puma/$2.rb"
				fi
		else
				echo "Starting $INSTANZ"
				bundle exec puma -C "config/puma/$2.rb"
		fi
		;;

'stop')
		if [ -f $Pidfile ] ; then
				echo "stopping $INSTANZ"
				bundle exec pumactl -P $Pidfile stop
		else
				echo "Cannot stop $INSTANZ - no Pidfile found!"
		fi
		;;

'restart')
    echo "restarting $INSTANZ"
    if test `ps -e | grep -c $Pid` = 1; then
				bundle exec pumactl -P $Pidfile restart
		else
				bundle exec puma -C "config/puma/$2.rb"
		fi
		;;

*)
		echo "usage: $0 { start | stop | restart } { production | development }"
		;;

esac
exit 0